/*
 *  I2Cで接続したPCF2129ATに
 *  日時とアラームの時刻を書き込み
 *  アラームの判定を割り込みピンで検知するプログラム
 *  Arduinoの2番ピンとPCF2129ATのINTピンを接続してください
 *  3番ピンをGNDに落とすことでアラーム状態を解除できます
 *  2015/11/30
 *  http://trac.switch-science.com/wiki/PCF2129AT
 */
#include <Wire.h>
#include "SPI.h"
#include "PCF2129AT.h"
#include <Adafruit_GFX.h>
#include <Adafruit_ILI9341.h>

//液晶用の設定
#define TFT_DC 9
#define TFT_CS 10
#define STMPE_CS 8



Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);
PCF2129AT rtc;
dt_dat dt,dt_leg;
int debug[0x19];
int legsec=0;
long nowcolor;

bool AlarmDetected = false;
void setup()
{
    pinMode(2,INPUT_PULLUP);// 割り込み用ピンの設定,内部プルアップ有効(~INTピンはオープンドレイン)
    pinMode(3,INPUT_PULLUP);// ボタン

    Serial.begin(9600);
    Wire.begin();
    rtc.init();
    tft.begin();
    tft.fillScreen(ILI9341_WHITE);
    tft.setTextColor(ILI9341_BLACK);

    Serial.println("PC2129AT Sample Program");


    /* 現在時刻の設定 */
    dt.y = 2015;                // 年の設定(2000年代)
    dt.wd = 5;                  // 曜日の設定(日:0-土:6)
    dt.mm = 11;                 // 月の設定(1-12)
    dt.d = 27;                   // 日の設定(1-)
    dt.h = 21;                   // 時の設定
    dt.m = 56;                  // 分の設定
    dt.s = 30;                   // 秒の設定
    rtc.setDate(&dt);              // 日時の設定

    /* アラーム日時の設定 */
    dt_dat dt_alarm;
    dt_alarm.wd = 5;                  // 曜日の設定(日:0-土:6)
    dt_alarm.d = 27;                   // 日の設定(1-)
    dt_alarm.h = 21;                   // 時の設定
    dt_alarm.m = 56;                  // 分の設定
    dt_alarm.s = 45;                   // 秒の設定

    /* 有効にするアラームの設定 */
    // デフォルト:無効
    rtc.enableAlarmSecond(); // 秒の一致を有効
    //rtc.enableAlarmMinute(); // 分の一致を有効
    //rtc.enableAlarmHour(); // 時の一致を有効
    //rtc.enableAlarmDay(); // 日にちの一致を有効
    //rtc.enableAlarmWeekDay(); // 曜日の一致を有効
    // 最後にsetAlarmをしてくだい(一致判定もこの時書き込まれます)
    rtc.setAlarm(&dt_alarm); // アラームの設定時刻の書き込み

    /* 有効にする割り込み許可の設定 */
    // 基本的にINTピンには各割り込みの論理和が適用されます
    // 利用したい割り込み機能だけ呼んでください
    rtc.enableInterruptAlarm();
    //rtc.enableInterruptTimeStamp();
    //rtc.enableInterruptSecMin();
    //rtc.enableInterruptWatchDog();
    //rtc.enableInterruptBattery();
    //rtc.enableInterruptLowBattery();
    // 設定後に書き込みます
    rtc.setInterruptsPIN();

    rtc.clearAF(); // アラームの割り込みフラグをクリア(初期化9
    //rtc.clearBLFlag();


    /* 外部ピン割り込みの設定 */
    attachInterrupt(0, detectAlarm, FALLING ); // デジタルピン2で検知
    attachInterrupt(1, clearAlarm, FALLING ); // デジタルピン3で検知

    tft.setTextSize(1);
    nowcolor = ILI9341_WHITE;
}

void loop() {

  rtc.getDate(&dt);          // 日時の取得

  // 日時の表示
  if(dt.s != legsec) // 秒に変化があれば描画
  {
    /* 前の描画を消去 */
    tft.setTextColor(nowcolor);
    showTimes(&dt_leg);
    /* 日時の表示 */
    tft.setTextColor(ILI9341_BLACK);
    showTimes(&dt);

    dt_leg = dt; // 前の値を更新

    /* アラーム検知されたら */
    if(AlarmDetected)
    {
      tft.setTextColor(ILI9341_RED);
      tft.setTextSize(3);
      tft.println("Alarm RINGING");
      rtc.clearAF(); // RTC側のフラグをクリア
    }
  }
  legsec = dt.s; // 前回読み取った病の値を更新

  delay(100);
}

/* 日時を表示 */
void showTimes(dt_dat *dtp)
{
  /* 日付 */
  tft.setCursor(30, 10);
  tft.setTextSize(2);
  tft.print(2000 + dtp->y,DEC);
  tft.print("/");
  tft.print(dtp->mm,DEC);
  tft.print("/");
  tft.print(dtp->d,DEC);
  tft.print("/");
  switch(dtp->wd)
  {
    case 0:
      tft.print("SUN");
      break;
    case 1:
      tft.print("MON");
      break;
    case 2:
      tft.print("TUE");
      break;
    case 3:
      tft.print("WED");
      break;
    case 4:
      tft.print("THU");
      break;
    case 5:
      tft.print("FRI");
      break;
    case 6:
      tft.print("SAT");
      break;
    default :
      break;
  }

  /* 時間 */
  tft.setCursor(0, 50);
  tft.setTextSize(5);
  tft.println(dtp->h,DEC);
  tft.println(dtp->m,DEC);
  tft.println(dtp->s,DEC);
}


/* ピン2の立ち下がりを検知次第実行される */
void detectAlarm()
{
  AlarmDetected = true;
  tft.fillScreen(ILI9341_GREEN);
  nowcolor = ILI9341_GREEN;
  // Arduinoの仕様上ここでWireインスタンスを利用できない
  // (割り込みの許可不許可が入れ込みハングアップする)
}

/* ピン3の立ち下がり検知次第実行される */
void clearAlarm()
{
  AlarmDetected = false;
  tft.fillScreen(ILI9341_WHITE);
  nowcolor = ILI9341_WHITE;
}
